// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: Ftrl.proto
// Protobuf C++ Version: 6.33.5

#ifndef Ftrl_2eproto_2epb_2eh
#define Ftrl_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 6033005
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/generated_enum_reflection.h"
#include "google/protobuf/unknown_field_set.h"
#include "Map.pb.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_Ftrl_2eproto

namespace google {
namespace protobuf {
namespace internal {
template <typename T>
::absl::string_view GetAnyMessageName();
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct TableStruct_Ftrl_2eproto {
  static const ::uint32_t offsets[];
};
extern "C" {
extern const ::google::protobuf::internal::DescriptorTable descriptor_table_Ftrl_2eproto;
}  // extern "C"
namespace mojo {
namespace spec {
enum FtrlModelType : int;
extern const uint32_t FtrlModelType_internal_data_[];
class FtrlOp;
struct FtrlOpDefaultTypeInternal;
extern FtrlOpDefaultTypeInternal _FtrlOp_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull FtrlOp_class_data_;
}  // namespace spec
}  // namespace mojo
namespace google {
namespace protobuf {
template <>
internal::EnumTraitsT<::mojo::spec::FtrlModelType_internal_data_>
    internal::EnumTraitsImpl::value<::mojo::spec::FtrlModelType>;
}  // namespace protobuf
}  // namespace google

namespace mojo {
namespace spec {
enum FtrlModelType : int {
  UNKNOWN_FTRL_MODEL_TYPE = 0,
  REGRESSION = 1,
  BINOMIAL = 2,
  MULTINOMIAL = 3,
  FtrlModelType_INT_MIN_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::min(),
  FtrlModelType_INT_MAX_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::max(),
};

extern const uint32_t FtrlModelType_internal_data_[];
inline constexpr FtrlModelType FtrlModelType_MIN =
    static_cast<FtrlModelType>(0);
inline constexpr FtrlModelType FtrlModelType_MAX =
    static_cast<FtrlModelType>(3);
inline bool FtrlModelType_IsValid(int value) {
  return 0 <= value && value <= 3;
}
inline constexpr int FtrlModelType_ARRAYSIZE = 3 + 1;
const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL FtrlModelType_descriptor();
template <typename T>
const ::std::string& FtrlModelType_Name(T value) {
  static_assert(::std::is_same<T, FtrlModelType>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to FtrlModelType_Name().");
  return FtrlModelType_Name(static_cast<FtrlModelType>(value));
}
template <>
inline const ::std::string& FtrlModelType_Name(FtrlModelType value) {
  return ::google::protobuf::internal::NameOfDenseEnum<FtrlModelType_descriptor, 0, 3>(
      static_cast<int>(value));
}
inline bool FtrlModelType_Parse(
    ::absl::string_view name, FtrlModelType* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<FtrlModelType>(FtrlModelType_descriptor(), name,
                                           value);
}

// ===================================================================


// -------------------------------------------------------------------

class FtrlOp final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:mojo.spec.FtrlOp) */ {
 public:
  inline FtrlOp() : FtrlOp(nullptr) {}
  ~FtrlOp() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(FtrlOp* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(FtrlOp));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR FtrlOp(::google::protobuf::internal::ConstantInitialized);

  inline FtrlOp(const FtrlOp& from) : FtrlOp(nullptr, from) {}
  inline FtrlOp(FtrlOp&& from) noexcept
      : FtrlOp(nullptr, ::std::move(from)) {}
  inline FtrlOp& operator=(const FtrlOp& from) {
    CopyFrom(from);
    return *this;
  }
  inline FtrlOp& operator=(FtrlOp&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const FtrlOp& default_instance() {
    return *reinterpret_cast<const FtrlOp*>(
        &_FtrlOp_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(FtrlOp& a, FtrlOp& b) { a.Swap(&b); }
  inline void Swap(FtrlOp* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(FtrlOp* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  FtrlOp* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<FtrlOp>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const FtrlOp& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const FtrlOp& from) { FtrlOp::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(FtrlOp* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "mojo.spec.FtrlOp"; }

  explicit FtrlOp(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  FtrlOp(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const FtrlOp& from);
  FtrlOp(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, FtrlOp&& from) noexcept
      : FtrlOp(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kInteractionsFieldNumber = 13,
    kDataLabelIdsFieldNumber = 14,
    kFeatureNamesFieldNumber = 16,
    kZFieldNumber = 11,
    kNFieldNumber = 12,
    kNlabelsFieldNumber = 1,
    kNfeaturesFieldNumber = 2,
    kAlphaFieldNumber = 4,
    kBetaFieldNumber = 5,
    kNinteractionsFieldNumber = 3,
    kModelTypeFieldNumber = 8,
    kLambda1FieldNumber = 6,
    kLambda2FieldNumber = 7,
    kMantissaNbitsFieldNumber = 9,
    kNbinsFieldNumber = 10,
    kModelNrowsFieldNumber = 15,
  };
  // repeated .mojo.spec.Int32Array interactions = 13;
  int interactions_size() const;
  private:
  int _internal_interactions_size() const;

  public:
  void clear_interactions() ;
  ::mojo::spec::Int32Array* PROTOBUF_NONNULL mutable_interactions(int index);
  ::google::protobuf::RepeatedPtrField<::mojo::spec::Int32Array>* PROTOBUF_NONNULL mutable_interactions();

  private:
  const ::google::protobuf::RepeatedPtrField<::mojo::spec::Int32Array>& _internal_interactions() const;
  ::google::protobuf::RepeatedPtrField<::mojo::spec::Int32Array>* PROTOBUF_NONNULL _internal_mutable_interactions();
  public:
  const ::mojo::spec::Int32Array& interactions(int index) const;
  ::mojo::spec::Int32Array* PROTOBUF_NONNULL add_interactions();
  const ::google::protobuf::RepeatedPtrField<::mojo::spec::Int32Array>& interactions() const;
  // repeated int32 data_label_ids = 14;
  int data_label_ids_size() const;
  private:
  int _internal_data_label_ids_size() const;

  public:
  void clear_data_label_ids() ;
  ::int32_t data_label_ids(int index) const;
  void set_data_label_ids(int index, ::int32_t value);
  void add_data_label_ids(::int32_t value);
  const ::google::protobuf::RepeatedField<::int32_t>& data_label_ids() const;
  ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL mutable_data_label_ids();

  private:
  const ::google::protobuf::RepeatedField<::int32_t>& _internal_data_label_ids() const;
  ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL _internal_mutable_data_label_ids();

  public:
  // repeated string feature_names = 16;
  int feature_names_size() const;
  private:
  int _internal_feature_names_size() const;

  public:
  void clear_feature_names() ;
  const ::std::string& feature_names(int index) const;
  ::std::string* PROTOBUF_NONNULL mutable_feature_names(int index);
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_feature_names(int index, Arg_&& value, Args_... args);
  ::std::string* PROTOBUF_NONNULL add_feature_names();
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void add_feature_names(Arg_&& value, Args_... args);
  const ::google::protobuf::RepeatedPtrField<::std::string>& feature_names() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL mutable_feature_names();

  private:
  const ::google::protobuf::RepeatedPtrField<::std::string>& _internal_feature_names() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL _internal_mutable_feature_names();

  public:
  // .mojo.spec.ExternalData z = 11;
  bool has_z() const;
  void clear_z() ;
  const ::mojo::spec::ExternalData& z() const;
  [[nodiscard]] ::mojo::spec::ExternalData* PROTOBUF_NULLABLE release_z();
  ::mojo::spec::ExternalData* PROTOBUF_NONNULL mutable_z();
  void set_allocated_z(::mojo::spec::ExternalData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_z(::mojo::spec::ExternalData* PROTOBUF_NULLABLE value);
  ::mojo::spec::ExternalData* PROTOBUF_NULLABLE unsafe_arena_release_z();

  private:
  const ::mojo::spec::ExternalData& _internal_z() const;
  ::mojo::spec::ExternalData* PROTOBUF_NONNULL _internal_mutable_z();

  public:
  // .mojo.spec.ExternalData n = 12;
  bool has_n() const;
  void clear_n() ;
  const ::mojo::spec::ExternalData& n() const;
  [[nodiscard]] ::mojo::spec::ExternalData* PROTOBUF_NULLABLE release_n();
  ::mojo::spec::ExternalData* PROTOBUF_NONNULL mutable_n();
  void set_allocated_n(::mojo::spec::ExternalData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_n(::mojo::spec::ExternalData* PROTOBUF_NULLABLE value);
  ::mojo::spec::ExternalData* PROTOBUF_NULLABLE unsafe_arena_release_n();

  private:
  const ::mojo::spec::ExternalData& _internal_n() const;
  ::mojo::spec::ExternalData* PROTOBUF_NONNULL _internal_mutable_n();

  public:
  // int32 nlabels = 1;
  void clear_nlabels() ;
  ::int32_t nlabels() const;
  void set_nlabels(::int32_t value);

  private:
  ::int32_t _internal_nlabels() const;
  void _internal_set_nlabels(::int32_t value);

  public:
  // int32 nfeatures = 2;
  void clear_nfeatures() ;
  ::int32_t nfeatures() const;
  void set_nfeatures(::int32_t value);

  private:
  ::int32_t _internal_nfeatures() const;
  void _internal_set_nfeatures(::int32_t value);

  public:
  // double alpha = 4;
  void clear_alpha() ;
  double alpha() const;
  void set_alpha(double value);

  private:
  double _internal_alpha() const;
  void _internal_set_alpha(double value);

  public:
  // double beta = 5;
  void clear_beta() ;
  double beta() const;
  void set_beta(double value);

  private:
  double _internal_beta() const;
  void _internal_set_beta(double value);

  public:
  // int32 ninteractions = 3;
  void clear_ninteractions() ;
  ::int32_t ninteractions() const;
  void set_ninteractions(::int32_t value);

  private:
  ::int32_t _internal_ninteractions() const;
  void _internal_set_ninteractions(::int32_t value);

  public:
  // .mojo.spec.FtrlModelType model_type = 8;
  void clear_model_type() ;
  ::mojo::spec::FtrlModelType model_type() const;
  void set_model_type(::mojo::spec::FtrlModelType value);

  private:
  ::mojo::spec::FtrlModelType _internal_model_type() const;
  void _internal_set_model_type(::mojo::spec::FtrlModelType value);

  public:
  // double lambda1 = 6;
  void clear_lambda1() ;
  double lambda1() const;
  void set_lambda1(double value);

  private:
  double _internal_lambda1() const;
  void _internal_set_lambda1(double value);

  public:
  // double lambda2 = 7;
  void clear_lambda2() ;
  double lambda2() const;
  void set_lambda2(double value);

  private:
  double _internal_lambda2() const;
  void _internal_set_lambda2(double value);

  public:
  // int32 mantissa_nbits = 9;
  void clear_mantissa_nbits() ;
  ::int32_t mantissa_nbits() const;
  void set_mantissa_nbits(::int32_t value);

  private:
  ::int32_t _internal_mantissa_nbits() const;
  void _internal_set_mantissa_nbits(::int32_t value);

  public:
  // int32 nbins = 10;
  void clear_nbins() ;
  ::int32_t nbins() const;
  void set_nbins(::int32_t value);

  private:
  ::int32_t _internal_nbins() const;
  void _internal_set_nbins(::int32_t value);

  public:
  // int32 model_nrows = 15;
  void clear_model_nrows() ;
  ::int32_t model_nrows() const;
  void set_model_nrows(::int32_t value);

  private:
  ::int32_t _internal_model_nrows() const;
  void _internal_set_model_nrows(::int32_t value);

  public:
  // @@protoc_insertion_point(class_scope:mojo.spec.FtrlOp)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<4, 16,
                                   3, 54,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const FtrlOp& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::mojo::spec::Int32Array > interactions_;
    ::google::protobuf::RepeatedField<::int32_t> data_label_ids_;
    ::google::protobuf::internal::CachedSize _data_label_ids_cached_byte_size_;
    ::google::protobuf::RepeatedPtrField<::std::string> feature_names_;
    ::mojo::spec::ExternalData* PROTOBUF_NULLABLE z_;
    ::mojo::spec::ExternalData* PROTOBUF_NULLABLE n_;
    ::int32_t nlabels_;
    ::int32_t nfeatures_;
    double alpha_;
    double beta_;
    ::int32_t ninteractions_;
    int model_type_;
    double lambda1_;
    double lambda2_;
    ::int32_t mantissa_nbits_;
    ::int32_t nbins_;
    ::int32_t model_nrows_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_Ftrl_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull FtrlOp_class_data_;

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// FtrlOp

// int32 nlabels = 1;
inline void FtrlOp::clear_nlabels() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.nlabels_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000020U);
}
inline ::int32_t FtrlOp::nlabels() const {
  // @@protoc_insertion_point(field_get:mojo.spec.FtrlOp.nlabels)
  return _internal_nlabels();
}
inline void FtrlOp::set_nlabels(::int32_t value) {
  _internal_set_nlabels(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:mojo.spec.FtrlOp.nlabels)
}
inline ::int32_t FtrlOp::_internal_nlabels() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.nlabels_;
}
inline void FtrlOp::_internal_set_nlabels(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.nlabels_ = value;
}

// int32 nfeatures = 2;
inline void FtrlOp::clear_nfeatures() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.nfeatures_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000040U);
}
inline ::int32_t FtrlOp::nfeatures() const {
  // @@protoc_insertion_point(field_get:mojo.spec.FtrlOp.nfeatures)
  return _internal_nfeatures();
}
inline void FtrlOp::set_nfeatures(::int32_t value) {
  _internal_set_nfeatures(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_set:mojo.spec.FtrlOp.nfeatures)
}
inline ::int32_t FtrlOp::_internal_nfeatures() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.nfeatures_;
}
inline void FtrlOp::_internal_set_nfeatures(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.nfeatures_ = value;
}

// int32 ninteractions = 3;
inline void FtrlOp::clear_ninteractions() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.ninteractions_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000200U);
}
inline ::int32_t FtrlOp::ninteractions() const {
  // @@protoc_insertion_point(field_get:mojo.spec.FtrlOp.ninteractions)
  return _internal_ninteractions();
}
inline void FtrlOp::set_ninteractions(::int32_t value) {
  _internal_set_ninteractions(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000200U);
  // @@protoc_insertion_point(field_set:mojo.spec.FtrlOp.ninteractions)
}
inline ::int32_t FtrlOp::_internal_ninteractions() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.ninteractions_;
}
inline void FtrlOp::_internal_set_ninteractions(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.ninteractions_ = value;
}

// double alpha = 4;
inline void FtrlOp::clear_alpha() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.alpha_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000080U);
}
inline double FtrlOp::alpha() const {
  // @@protoc_insertion_point(field_get:mojo.spec.FtrlOp.alpha)
  return _internal_alpha();
}
inline void FtrlOp::set_alpha(double value) {
  _internal_set_alpha(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000080U);
  // @@protoc_insertion_point(field_set:mojo.spec.FtrlOp.alpha)
}
inline double FtrlOp::_internal_alpha() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.alpha_;
}
inline void FtrlOp::_internal_set_alpha(double value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.alpha_ = value;
}

// double beta = 5;
inline void FtrlOp::clear_beta() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.beta_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000100U);
}
inline double FtrlOp::beta() const {
  // @@protoc_insertion_point(field_get:mojo.spec.FtrlOp.beta)
  return _internal_beta();
}
inline void FtrlOp::set_beta(double value) {
  _internal_set_beta(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000100U);
  // @@protoc_insertion_point(field_set:mojo.spec.FtrlOp.beta)
}
inline double FtrlOp::_internal_beta() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.beta_;
}
inline void FtrlOp::_internal_set_beta(double value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.beta_ = value;
}

// double lambda1 = 6;
inline void FtrlOp::clear_lambda1() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.lambda1_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000800U);
}
inline double FtrlOp::lambda1() const {
  // @@protoc_insertion_point(field_get:mojo.spec.FtrlOp.lambda1)
  return _internal_lambda1();
}
inline void FtrlOp::set_lambda1(double value) {
  _internal_set_lambda1(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000800U);
  // @@protoc_insertion_point(field_set:mojo.spec.FtrlOp.lambda1)
}
inline double FtrlOp::_internal_lambda1() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.lambda1_;
}
inline void FtrlOp::_internal_set_lambda1(double value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.lambda1_ = value;
}

// double lambda2 = 7;
inline void FtrlOp::clear_lambda2() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.lambda2_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00001000U);
}
inline double FtrlOp::lambda2() const {
  // @@protoc_insertion_point(field_get:mojo.spec.FtrlOp.lambda2)
  return _internal_lambda2();
}
inline void FtrlOp::set_lambda2(double value) {
  _internal_set_lambda2(value);
  SetHasBit(_impl_._has_bits_[0], 0x00001000U);
  // @@protoc_insertion_point(field_set:mojo.spec.FtrlOp.lambda2)
}
inline double FtrlOp::_internal_lambda2() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.lambda2_;
}
inline void FtrlOp::_internal_set_lambda2(double value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.lambda2_ = value;
}

// .mojo.spec.FtrlModelType model_type = 8;
inline void FtrlOp::clear_model_type() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.model_type_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000400U);
}
inline ::mojo::spec::FtrlModelType FtrlOp::model_type() const {
  // @@protoc_insertion_point(field_get:mojo.spec.FtrlOp.model_type)
  return _internal_model_type();
}
inline void FtrlOp::set_model_type(::mojo::spec::FtrlModelType value) {
  _internal_set_model_type(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000400U);
  // @@protoc_insertion_point(field_set:mojo.spec.FtrlOp.model_type)
}
inline ::mojo::spec::FtrlModelType FtrlOp::_internal_model_type() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::mojo::spec::FtrlModelType>(_impl_.model_type_);
}
inline void FtrlOp::_internal_set_model_type(::mojo::spec::FtrlModelType value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.model_type_ = value;
}

// int32 mantissa_nbits = 9;
inline void FtrlOp::clear_mantissa_nbits() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.mantissa_nbits_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00002000U);
}
inline ::int32_t FtrlOp::mantissa_nbits() const {
  // @@protoc_insertion_point(field_get:mojo.spec.FtrlOp.mantissa_nbits)
  return _internal_mantissa_nbits();
}
inline void FtrlOp::set_mantissa_nbits(::int32_t value) {
  _internal_set_mantissa_nbits(value);
  SetHasBit(_impl_._has_bits_[0], 0x00002000U);
  // @@protoc_insertion_point(field_set:mojo.spec.FtrlOp.mantissa_nbits)
}
inline ::int32_t FtrlOp::_internal_mantissa_nbits() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.mantissa_nbits_;
}
inline void FtrlOp::_internal_set_mantissa_nbits(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.mantissa_nbits_ = value;
}

// int32 nbins = 10;
inline void FtrlOp::clear_nbins() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.nbins_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00004000U);
}
inline ::int32_t FtrlOp::nbins() const {
  // @@protoc_insertion_point(field_get:mojo.spec.FtrlOp.nbins)
  return _internal_nbins();
}
inline void FtrlOp::set_nbins(::int32_t value) {
  _internal_set_nbins(value);
  SetHasBit(_impl_._has_bits_[0], 0x00004000U);
  // @@protoc_insertion_point(field_set:mojo.spec.FtrlOp.nbins)
}
inline ::int32_t FtrlOp::_internal_nbins() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.nbins_;
}
inline void FtrlOp::_internal_set_nbins(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.nbins_ = value;
}

// .mojo.spec.ExternalData z = 11;
inline bool FtrlOp::has_z() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000008U);
  PROTOBUF_ASSUME(!value || _impl_.z_ != nullptr);
  return value;
}
inline const ::mojo::spec::ExternalData& FtrlOp::_internal_z() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::mojo::spec::ExternalData* p = _impl_.z_;
  return p != nullptr ? *p : reinterpret_cast<const ::mojo::spec::ExternalData&>(::mojo::spec::_ExternalData_default_instance_);
}
inline const ::mojo::spec::ExternalData& FtrlOp::z() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:mojo.spec.FtrlOp.z)
  return _internal_z();
}
inline void FtrlOp::unsafe_arena_set_allocated_z(
    ::mojo::spec::ExternalData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.z_);
  }
  _impl_.z_ = reinterpret_cast<::mojo::spec::ExternalData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:mojo.spec.FtrlOp.z)
}
inline ::mojo::spec::ExternalData* PROTOBUF_NULLABLE FtrlOp::release_z() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::mojo::spec::ExternalData* released = _impl_.z_;
  _impl_.z_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::mojo::spec::ExternalData* PROTOBUF_NULLABLE FtrlOp::unsafe_arena_release_z() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:mojo.spec.FtrlOp.z)

  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::mojo::spec::ExternalData* temp = _impl_.z_;
  _impl_.z_ = nullptr;
  return temp;
}
inline ::mojo::spec::ExternalData* PROTOBUF_NONNULL FtrlOp::_internal_mutable_z() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.z_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::mojo::spec::ExternalData>(GetArena());
    _impl_.z_ = reinterpret_cast<::mojo::spec::ExternalData*>(p);
  }
  return _impl_.z_;
}
inline ::mojo::spec::ExternalData* PROTOBUF_NONNULL FtrlOp::mutable_z()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::mojo::spec::ExternalData* _msg = _internal_mutable_z();
  // @@protoc_insertion_point(field_mutable:mojo.spec.FtrlOp.z)
  return _msg;
}
inline void FtrlOp::set_allocated_z(::mojo::spec::ExternalData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.z_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }

  _impl_.z_ = reinterpret_cast<::mojo::spec::ExternalData*>(value);
  // @@protoc_insertion_point(field_set_allocated:mojo.spec.FtrlOp.z)
}

// .mojo.spec.ExternalData n = 12;
inline bool FtrlOp::has_n() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000010U);
  PROTOBUF_ASSUME(!value || _impl_.n_ != nullptr);
  return value;
}
inline const ::mojo::spec::ExternalData& FtrlOp::_internal_n() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::mojo::spec::ExternalData* p = _impl_.n_;
  return p != nullptr ? *p : reinterpret_cast<const ::mojo::spec::ExternalData&>(::mojo::spec::_ExternalData_default_instance_);
}
inline const ::mojo::spec::ExternalData& FtrlOp::n() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:mojo.spec.FtrlOp.n)
  return _internal_n();
}
inline void FtrlOp::unsafe_arena_set_allocated_n(
    ::mojo::spec::ExternalData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.n_);
  }
  _impl_.n_ = reinterpret_cast<::mojo::spec::ExternalData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:mojo.spec.FtrlOp.n)
}
inline ::mojo::spec::ExternalData* PROTOBUF_NULLABLE FtrlOp::release_n() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::mojo::spec::ExternalData* released = _impl_.n_;
  _impl_.n_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::mojo::spec::ExternalData* PROTOBUF_NULLABLE FtrlOp::unsafe_arena_release_n() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:mojo.spec.FtrlOp.n)

  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::mojo::spec::ExternalData* temp = _impl_.n_;
  _impl_.n_ = nullptr;
  return temp;
}
inline ::mojo::spec::ExternalData* PROTOBUF_NONNULL FtrlOp::_internal_mutable_n() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.n_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::mojo::spec::ExternalData>(GetArena());
    _impl_.n_ = reinterpret_cast<::mojo::spec::ExternalData*>(p);
  }
  return _impl_.n_;
}
inline ::mojo::spec::ExternalData* PROTOBUF_NONNULL FtrlOp::mutable_n()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::mojo::spec::ExternalData* _msg = _internal_mutable_n();
  // @@protoc_insertion_point(field_mutable:mojo.spec.FtrlOp.n)
  return _msg;
}
inline void FtrlOp::set_allocated_n(::mojo::spec::ExternalData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.n_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  }

  _impl_.n_ = reinterpret_cast<::mojo::spec::ExternalData*>(value);
  // @@protoc_insertion_point(field_set_allocated:mojo.spec.FtrlOp.n)
}

// repeated .mojo.spec.Int32Array interactions = 13;
inline int FtrlOp::_internal_interactions_size() const {
  return _internal_interactions().size();
}
inline int FtrlOp::interactions_size() const {
  return _internal_interactions_size();
}
inline ::mojo::spec::Int32Array* PROTOBUF_NONNULL FtrlOp::mutable_interactions(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:mojo.spec.FtrlOp.interactions)
  return _internal_mutable_interactions()->Mutable(index);
}
inline ::google::protobuf::RepeatedPtrField<::mojo::spec::Int32Array>* PROTOBUF_NONNULL FtrlOp::mutable_interactions()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:mojo.spec.FtrlOp.interactions)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_interactions();
}
inline const ::mojo::spec::Int32Array& FtrlOp::interactions(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:mojo.spec.FtrlOp.interactions)
  return _internal_interactions().Get(index);
}
inline ::mojo::spec::Int32Array* PROTOBUF_NONNULL FtrlOp::add_interactions()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::mojo::spec::Int32Array* _add =
      _internal_mutable_interactions()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:mojo.spec.FtrlOp.interactions)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::mojo::spec::Int32Array>& FtrlOp::interactions() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:mojo.spec.FtrlOp.interactions)
  return _internal_interactions();
}
inline const ::google::protobuf::RepeatedPtrField<::mojo::spec::Int32Array>&
FtrlOp::_internal_interactions() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.interactions_;
}
inline ::google::protobuf::RepeatedPtrField<::mojo::spec::Int32Array>* PROTOBUF_NONNULL
FtrlOp::_internal_mutable_interactions() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.interactions_;
}

// repeated int32 data_label_ids = 14;
inline int FtrlOp::_internal_data_label_ids_size() const {
  return _internal_data_label_ids().size();
}
inline int FtrlOp::data_label_ids_size() const {
  return _internal_data_label_ids_size();
}
inline void FtrlOp::clear_data_label_ids() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.data_label_ids_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000002U);
}
inline ::int32_t FtrlOp::data_label_ids(int index) const {
  // @@protoc_insertion_point(field_get:mojo.spec.FtrlOp.data_label_ids)
  return _internal_data_label_ids().Get(index);
}
inline void FtrlOp::set_data_label_ids(int index, ::int32_t value) {
  _internal_mutable_data_label_ids()->Set(index, value);
  // @@protoc_insertion_point(field_set:mojo.spec.FtrlOp.data_label_ids)
}
inline void FtrlOp::add_data_label_ids(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _internal_mutable_data_label_ids()->Add(value);
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_add:mojo.spec.FtrlOp.data_label_ids)
}
inline const ::google::protobuf::RepeatedField<::int32_t>& FtrlOp::data_label_ids() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:mojo.spec.FtrlOp.data_label_ids)
  return _internal_data_label_ids();
}
inline ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL FtrlOp::mutable_data_label_ids()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_mutable_list:mojo.spec.FtrlOp.data_label_ids)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_data_label_ids();
}
inline const ::google::protobuf::RepeatedField<::int32_t>&
FtrlOp::_internal_data_label_ids() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.data_label_ids_;
}
inline ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL
FtrlOp::_internal_mutable_data_label_ids() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.data_label_ids_;
}

// int32 model_nrows = 15;
inline void FtrlOp::clear_model_nrows() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.model_nrows_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00008000U);
}
inline ::int32_t FtrlOp::model_nrows() const {
  // @@protoc_insertion_point(field_get:mojo.spec.FtrlOp.model_nrows)
  return _internal_model_nrows();
}
inline void FtrlOp::set_model_nrows(::int32_t value) {
  _internal_set_model_nrows(value);
  SetHasBit(_impl_._has_bits_[0], 0x00008000U);
  // @@protoc_insertion_point(field_set:mojo.spec.FtrlOp.model_nrows)
}
inline ::int32_t FtrlOp::_internal_model_nrows() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.model_nrows_;
}
inline void FtrlOp::_internal_set_model_nrows(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.model_nrows_ = value;
}

// repeated string feature_names = 16;
inline int FtrlOp::_internal_feature_names_size() const {
  return _internal_feature_names().size();
}
inline int FtrlOp::feature_names_size() const {
  return _internal_feature_names_size();
}
inline void FtrlOp::clear_feature_names() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.feature_names_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000004U);
}
inline ::std::string* PROTOBUF_NONNULL FtrlOp::add_feature_names()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::std::string* _s =
      _internal_mutable_feature_names()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_add_mutable:mojo.spec.FtrlOp.feature_names)
  return _s;
}
inline const ::std::string& FtrlOp::feature_names(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:mojo.spec.FtrlOp.feature_names)
  return _internal_feature_names().Get(index);
}
inline ::std::string* PROTOBUF_NONNULL FtrlOp::mutable_feature_names(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:mojo.spec.FtrlOp.feature_names)
  return _internal_mutable_feature_names()->Mutable(index);
}
template <typename Arg_, typename... Args_>
inline void FtrlOp::set_feature_names(int index, Arg_&& value, Args_... args) {
  ::google::protobuf::internal::AssignToString(*_internal_mutable_feature_names()->Mutable(index), ::std::forward<Arg_>(value),
                        args... );
  // @@protoc_insertion_point(field_set:mojo.spec.FtrlOp.feature_names)
}
template <typename Arg_, typename... Args_>
inline void FtrlOp::add_feature_names(Arg_&& value, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::google::protobuf::internal::AddToRepeatedPtrField(
      ::google::protobuf::MessageLite::internal_visibility(), GetArena(),
      *_internal_mutable_feature_names(), ::std::forward<Arg_>(value),
      args... );
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_add:mojo.spec.FtrlOp.feature_names)
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>& FtrlOp::feature_names()
    const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:mojo.spec.FtrlOp.feature_names)
  return _internal_feature_names();
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
FtrlOp::mutable_feature_names() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_mutable_list:mojo.spec.FtrlOp.feature_names)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_feature_names();
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>&
FtrlOp::_internal_feature_names() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.feature_names_;
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
FtrlOp::_internal_mutable_feature_names() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.feature_names_;
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace spec
}  // namespace mojo


namespace google {
namespace protobuf {

template <>
struct is_proto_enum<::mojo::spec::FtrlModelType> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::mojo::spec::FtrlModelType>() {
  return ::mojo::spec::FtrlModelType_descriptor();
}

}  // namespace protobuf
}  // namespace google

// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"

#endif  // Ftrl_2eproto_2epb_2eh
